--------------------------------------------------------------------------------
-- Satchel Charge by Holzchopf
-- v1.2 - 07 Oct 2011	- Info text added
-- v1.1 - 28 Sep 2011	- Updated hierarchy
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.satchelcharge={}
holzchopf.satchelcharge.charge={}

-- load ressources
holzchopf.satchelcharge.gfx_charge=loadgfx("holzchopf/satchelcharge.png")
setmidhandle(holzchopf.satchelcharge.gfx_charge)
holzchopf.satchelcharge.gfx_wpn=loadgfx("weapons/rc.bmp")
setmidhandle(holzchopf.satchelcharge.gfx_wpn)
holzchopf.satchelcharge.sfx_att=loadsfx("throw.ogg")
holzchopf.satchelcharge.sfx_bounce=loadsfx("bounce.wav")
holzchopf.satchelcharge.sfx_steam=loadsfx("steam.ogg")

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.satchelcharge.id=addweapon("holzchopf.satchelcharge","Satchel Charge",holzchopf.satchelcharge.gfx_charge,0,0)
holzchopf.satchelcharge.ammo=1

function holzchopf.satchelcharge.draw()
	if weapon_shots<holzchopf.satchelcharge.ammo then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		setscale(1,1)
		setrotation(0)
		drawimage(holzchopf.satchelcharge.gfx_charge,getplayerx(0)+getplayerdirection(0)*7,getplayery(0)+2)
		hudammobar(holzchopf.satchelcharge.ammo-weapon_shots,holzchopf.satchelcharge.ammo)
	elseif weapon_shots==holzchopf.satchelcharge.ammo then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		drawinhand(holzchopf.satchelcharge.gfx_wpn,7,0)
		hudinfo("Hit [attack] to detonate the charge!")
	end
end

function holzchopf.satchelcharge.attack(attack)
	if attack==1 and keyup==1 then
		keyup=0
		if weapon_shots<holzchopf.satchelcharge.ammo then
			-- use
			useweapon(0)
			playsound(holzchopf.satchelcharge.sfx_att)
			weapon_shots=weapon_shots+1
			-- throw satchel
			id=createprojectile(holzchopf.satchelcharge.charge.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+getplayerdirection(0)*7
			projectiles[id].y=getplayery(0)+2
			-- Set speed of projectile
			projectiles[id].sx=getplayerdirection(0)*2
			projectiles[id].sy=-1
			projectiles[id].timer=getframe()+25
		else
			-- detonate
			weapon_timer=-1
			weapon_shots=weapon_shots+1
			endturn()
		end
	end
	if attack==0 then keyup=1 end
end

--------------------------------------------------------------------------------
-- Projectile
--------------------------------------------------------------------------------
holzchopf.satchelcharge.charge.id=addprojectile("holzchopf.satchelcharge.charge")

function holzchopf.satchelcharge.charge.draw(id)
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	setrotation(0)
	drawimage(holzchopf.satchelcharge.gfx_charge,projectiles[id].x,projectiles[id].y)
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
	-- blinking light
	if getframe()>=projectiles[id].timer then
		projectiles[id].timer=getframe()+25
		particle(p_muzzle,projectiles[id].x-4,projectiles[id].y-3)
		particlecolor(255,0,0)
		particlesize(0.5,0.5)
		particlefadealpha(0.1)
	end
	-- out of time
	if getframesleft()<=0 then
		particle(p_smoke,projectiles[id].x,projectiles[id].y)
		particlesize(10,10)
		particlefadealpha(0.01)
		playsound(holzchopf.satchelcharge.sfx_steam)
		freeprojectile(id)
	end
end

function holzchopf.satchelcharge.charge.update(id)
	-- Gravity influence on speed + decrease x speed
	projectiles[id].sx=projectiles[id].sx*0.95
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- Move X
		projectiles[id].x=projectiles[id].x+msubx
		if collision(holzchopf.satchelcharge.gfx_charge,projectiles[id].x,projectiles[id].y,1,1)==1 then
			if terraincollision()==1 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sx)>0.5) then playsound(holzchopf.satchelcharge.sfx_bounce) end
				projectiles[id].x=projectiles[id].x-msubx
				projectiles[id].sx=-projectiles[id].sx*0.05
				msubx=-msubx*0.05
			end
		else
			projectiles[id].ignore=0
		end
		-- Move Y
		projectiles[id].y=projectiles[id].y+msuby
		if collision(holzchopf.satchelcharge.gfx_charge,projectiles[id].x,projectiles[id].y,1,1)==1 then
			if terraincollision()==1 or playercollision()~=projectiles[id].ignore then
				if (math.abs(projectiles[id].sy)>0.5) then playsound(holzchopf.satchelcharge.sfx_bounce) end
				projectiles[id].y=projectiles[id].y-msuby
				projectiles[id].sy=-projectiles[id].sy*0.3
				msuby=-msuby*0.3
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	-- detonate
	if weapon_timer<0 then
		-- Cause damage
		arealdamage(projectiles[id].x,projectiles[id].y,125,65)
		-- Destroy terrain
		terrainexplosion(projectiles[id].x,projectiles[id].y,55,1)
		-- Crater
		grey=math.random(0,40)
		if math.random(0,1)==1 then
			terrainalphaimage(gfx_crater150,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		else
			terrainalphaimage(gfx_crater175,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
		end
		freeprojectile(id)
	end
end